# iparapheur-provisioning
iparapheur v5.x main core application.

The main link between every sub-services, integrating business code logic.


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: DEVELOP
- Package version: 1.5.2
- Build package: org.openapitools.codegen.languages.PythonClientCodegen
For more information, please visit [https://libriciel.fr](https://libriciel.fr)

## Requirements.

Python &gt;&#x3D;3.7

## Migration from other generators like python and python-legacy

### Changes
1. This generator uses spec case for all (object) property names and parameter names.
    - So if the spec has a property name like camelCase, it will use camelCase rather than camel_case
    - So you will need to update how you input and read properties to use spec case
2. Endpoint parameters are stored in dictionaries to prevent collisions (explanation below)
    - So you will need to update how you pass data in to endpoints
3. Endpoint responses now include the original response, the deserialized response body, and (todo)the deserialized headers
    - So you will need to update your code to use response.body to access deserialized data
4. All validated data is instantiated in an instance that subclasses all validated Schema classes and Decimal/str/list/tuple/frozendict/NoneClass/BoolClass/bytes/io.FileIO
    - This means that you can use isinstance to check if a payload validated against a schema class
    - This means that no data will be of type None/True/False
        - ingested None will subclass NoneClass
        - ingested True will subclass BoolClass
        - ingested False will subclass BoolClass
        - So if you need to check is True/False/None, instead use instance.is_true_oapg()/.is_false_oapg()/.is_none_oapg()
5. All validated class instances are immutable except for ones based on io.File
    - This is because if properties were changed after validation, that validation would no longer apply
    - So no changing values or property values after a class has been instantiated
6. String + Number types with formats
    - String type data is stored as a string and if you need to access types based on its format like date,
    date-time, uuid, number etc then you will need to use accessor functions on the instance
    - type string + format: See .as_date_oapg, .as_datetime_oapg, .as_decimal_oapg, .as_uuid_oapg
    - type number + format: See .as_float_oapg, .as_int_oapg
    - this was done because openapi/json-schema defines constraints. string data may be type string with no format
    keyword in one schema, and include a format constraint in another schema
    - So if you need to access a string format based type, use as_date_oapg/as_datetime_oapg/as_decimal_oapg/as_uuid_oapg
    - So if you need to access a number format based type, use as_int_oapg/as_float_oapg
7. Property access on AnyType(type unset) or object(dict) schemas
    - Only required keys with valid python names are properties like .someProp and have type hints
    - All optional keys may not exist, so properties are not defined for them
    - One can access optional values with dict_instance['optionalProp'] and KeyError will be raised if it does not exist
    - Use get_item_oapg if you need a way to always get a value whether or not the key exists
        - If the key does not exist, schemas.unset is returned from calling dict_instance.get_item_oapg('optionalProp')
        - All required and optional keys have type hints for this method, and @typing.overload is used
        - A type hint is also generated for additionalProperties accessed using this method
    - So you will need to update you code to use some_instance['optionalProp'] to access optional property
    and additionalProperty values
8. The location of the api classes has changed
    - Api classes are located in your_package.apis.tags.some_api
    - This change was made to eliminate redundant code generation
    - Legacy generators generated the same endpoint twice if it had > 1 tag on it
    - This generator defines an endpoint in one class, then inherits that class to generate
      apis by tags and by paths
    - This change reduces code and allows quicker run time if you use the path apis
        - path apis are at your_package.apis.paths.some_path
    - Those apis will only load their needed models, which is less to load than all of the resources needed in a tag api
    - So you will need to update your import paths to the api classes

### Why are Oapg and _oapg used in class and method names?
Classes can have arbitrarily named properties set on them
Endpoints can have arbitrary operationId method names set
For those reasons, I use the prefix Oapg and _oapg to greatly reduce the likelihood of collisions
on protected + public classes/methods.
oapg stands for OpenApi Python Generator.

### Object property spec case
This was done because when payloads are ingested, they can be validated against N number of schemas.
If the input signature used a different property name then that has mutated the payload.
So SchemaA and SchemaB must both see the camelCase spec named variable.
Also it is possible to send in two properties, named camelCase and camel_case in the same payload.
That use case should be support so spec case is used.

### Parameter spec case
Parameters can be included in different locations including:
- query
- path
- header
- cookie

Any of those parameters could use the same parameter names, so if every parameter
was included as an endpoint parameter in a function signature, they would collide.
For that reason, each of those inputs have been separated out into separate typed dictionaries:
- query_params
- path_params
- header_params
- cookie_params

So when updating your code, you will need to pass endpoint parameters in using those
dictionaries.

### Endpoint responses
Endpoint responses have been enriched to now include more information.
Any response reom an endpoint will now include the following properties:
response: urllib3.HTTPResponse
body: typing.Union[Unset, Schema]
headers: typing.Union[Unset, TODO]
Note: response header deserialization has not yet been added


## Installation & Usage
### pip install

If the python package is hosted on a repository, you can install directly using:

```sh
pip install git+https://github.com/GIT_USER_ID/GIT_REPO_ID.git
```
(you may need to run `pip` with root permission: `sudo pip install git+https://github.com/GIT_USER_ID/GIT_REPO_ID.git`)

Then import the package:
```python
import iparapheur_provisioning
```

### Setuptools

Install via [Setuptools](http://pypi.python.org/pypi/setuptools).

```sh
python setup.py install --user
```
(or `sudo python setup.py install` to install the package for all users)

Then import the package:
```python
import iparapheur_provisioning
```

## Getting Started

Please follow the [installation procedure](#installation--usage) and then run the following:

```python

import time
import iparapheur_provisioning
from pprint import pprint
from iparapheur_provisioning.apis.tags import admin_desk_api
from iparapheur_provisioning.model.desk_dto import DeskDto
from iparapheur_provisioning.model.error_response import ErrorResponse
# Defining the host is optional and defaults to http://localhost:8080
# See configuration.py for a list of all supported configuration parameters.
configuration = iparapheur_provisioning.Configuration(
    host = "http://localhost:8080"
)

# The client must configure the authentication and authorization parameters
# in accordance with the API server security policy.
# Examples for each auth method are provided below, use the example that
# satisfies your auth use case.

# Configure OAuth2 access token for authorization: spring_oauth
configuration = iparapheur_provisioning.Configuration(
    host = "http://localhost:8080",
    access_token = 'YOUR_ACCESS_TOKEN'
)

# Enter a context with an instance of the API client
with iparapheur_provisioning.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = admin_desk_api.AdminDeskApi(api_client)
    tenant_id = "tenantId_example" # str | Tenant id
desk_dto = DeskDto(
        id="id_example",
        name="name_example",
        tenant_id="tenant_id_example",
        short_name="short_name_example",
        description="description_example",
        folder_creation_allowed=True,
        action_allowed=True,
        archiving_allowed=True,
        chain_allowed=True,
        parent_desk_id="parent_desk_id_example",
        parent_desk=DeskRepresentation(
            id="id_example",
            name="name_example",
            tenant_id="tenant_id_example",
        ),
        owner_ids=[
            "owner_ids_example"
        ],
        owners=[
            UserRepresentation(
                id="id_example",
                user_name="user_name_example",
                first_name="first_name_example",
                last_name="last_name_example",
                email="email_example",
                privilege=UserPrivilege("NONE"),
                is_ldap_synchronized=True,
            )
        ],
,
        associated_desks=[
            DeskRepresentation()
        ],
,
        filterable_metadata=[
            MetadataRepresentation(
                id="id_example",
                name="name_example",
                key="key_example",
                index=1,
                type=MetadataType("TEXT"),
            )
        ],
,
        filterable_subtypes=[
            TypologyRepresentation(
                id="id_example",
                name="name_example",
                parent_id="parent_id_example",
                children_count=1,
            )
        ],
,
,
,
,
,
,
    ) # DeskDto | 

    try:
        # Create a desk
        api_response = api_instance.create_desk(tenant_iddesk_dto)
        pprint(api_response)
    except iparapheur_provisioning.ApiException as e:
        print("Exception when calling AdminDeskApi->create_desk: %s\n" % e)
```

## Documentation for API Endpoints

All URIs are relative to *http://localhost:8080*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*AdminDeskApi* | [**create_desk**](docs/apis/tags/AdminDeskApi.md#create_desk) | **post** /api/provisioning/v1/admin/tenant/{tenantId}/desk | Create a desk
*AdminDeskApi* | [**delete_desk**](docs/apis/tags/AdminDeskApi.md#delete_desk) | **delete** /api/provisioning/v1/admin/tenant/{tenantId}/desk/{deskId} | Delete desk
*AdminDeskApi* | [**edit_desk**](docs/apis/tags/AdminDeskApi.md#edit_desk) | **put** /api/provisioning/v1/admin/tenant/{tenantId}/desk/{deskId} | Edit a desk
*AdminDeskApi* | [**get_desk_as_admin**](docs/apis/tags/AdminDeskApi.md#get_desk_as_admin) | **get** /api/provisioning/v1/admin/tenant/{tenantId}/desk/{deskId} | Get a full desk description
*AdminTenantApi* | [**create_tenant**](docs/apis/tags/AdminTenantApi.md#create_tenant) | **post** /api/provisioning/v1/admin/tenant | Create a tenant
*AdminTenantApi* | [**delete_tenant**](docs/apis/tags/AdminTenantApi.md#delete_tenant) | **delete** /api/provisioning/v1/admin/tenant/{tenantId} | Delete a tenant
*AdminTenantApi* | [**get_tenant**](docs/apis/tags/AdminTenantApi.md#get_tenant) | **get** /api/provisioning/v1/admin/tenant/{tenantId} | Get a tenant
*AdminTenantApi* | [**update_tenant**](docs/apis/tags/AdminTenantApi.md#update_tenant) | **put** /api/provisioning/v1/admin/tenant/{tenantId} | Edit a tenant

## Documentation For Models

 - [DelegationSortBy](docs/models/DelegationSortBy.md)
 - [DeskDto](docs/models/DeskDto.md)
 - [DeskRepresentation](docs/models/DeskRepresentation.md)
 - [ErrorResponse](docs/models/ErrorResponse.md)
 - [ExternalSignatureConfigSortBy](docs/models/ExternalSignatureConfigSortBy.md)
 - [FolderSortBy](docs/models/FolderSortBy.md)
 - [InternalMetadata](docs/models/InternalMetadata.md)
 - [LayerSortBy](docs/models/LayerSortBy.md)
 - [MetadataRepresentation](docs/models/MetadataRepresentation.md)
 - [MetadataSortBy](docs/models/MetadataSortBy.md)
 - [MetadataType](docs/models/MetadataType.md)
 - [SealCertificateSortBy](docs/models/SealCertificateSortBy.md)
 - [TenantDto](docs/models/TenantDto.md)
 - [TenantSortBy](docs/models/TenantSortBy.md)
 - [TypologyRepresentation](docs/models/TypologyRepresentation.md)
 - [TypologySortBy](docs/models/TypologySortBy.md)
 - [UserPrivilege](docs/models/UserPrivilege.md)
 - [UserRepresentation](docs/models/UserRepresentation.md)
 - [UserSortBy](docs/models/UserSortBy.md)
 - [WorkflowDefinitionSortBy](docs/models/WorkflowDefinitionSortBy.md)

## Documentation For Authorization

 Authentication schemes defined for the API:
## spring_oauth

- **Type**: OAuth
- **Flow**: accessCode
- **Authorization URL**: ./auth/realms/api/protocol/openid-connect/auth
- **Scopes**: N/A


## Author

iparapheur@libriciel.coop
iparapheur@libriciel.coop

## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in iparapheur_provisioning.apis and iparapheur_provisioning.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from iparapheur_provisioning.apis.default_api import DefaultApi`
- `from iparapheur_provisioning.model.pet import Pet`

Solution 1:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import iparapheur_provisioning
from iparapheur_provisioning.apis import *
from iparapheur_provisioning.models import *
```
